#!/bin/sh

# This script generates Alpine images for OpenNebula.
#
# Test image locally (without network) with:
#   qemu-system-x86_64 -enable-kvm -m 1G -drive file=$IMAGE,format=qcow2

set -e
set -x

# XXX: Handle command-line arguments?
RELEASE=v3.14
ARCH=x86_64
IMAGE_PATH=alpine-$RELEASE-$(date -I).img.qcow2
IMAGE_SIZE=10G
NBD_DEVICE=/dev/nbd0
APK_MIRROR=http://dl-2.alpinelinux.org/alpine/ # Mind the trailing /

ONE_CONTEXT_APK_URL="https://github.com/OpenNebula/addon-context-linux/releases/download/v5.10.0/one-context-5.10.0-r1.apk"
ONE_CONTEXT_APK_PATH=/root/one-context.apk

cleanup() {
	# The order here is important.
	umount /mnt/dev/pts 2>/dev/null || true
	umount /mnt/dev/shm 2>/dev/null || true
	umount /mnt/dev 2>/dev/null || true
	umount /mnt/proc 2>/dev/null || true
	umount /mnt/run 2>/dev/null || true
	umount /mnt/sys 2>/dev/null || true
	umount /mnt/boot 2>/dev/null || true
	umount /mnt 2>/dev/null || true
	qemu-nbd --disconnect "$NBD_DEVICE" || true
}

run_root() {
	chroot /mnt /usr/bin/env \
		PATH=/sbin:/usr/sbin:/bin:/usr/bin \
		sh -c "$*"
}

if [ "$(whoami)" != 'root' ]; then
	echo "This script must be run as root." >&2
	exit 1
fi

if [ "$(lsb_release --short --id)" != "Alpine" ]; then
	echo "WARNING: this script has been designed to run on an Alpine system." >&2
	echo "WARNING: Not running Alpine. Giving you 5 seconds to abort." >&2
	sleep 5
fi

# Create base QCOW2 image.
qemu-img create -f qcow2 "$IMAGE_PATH" "$IMAGE_SIZE"
modprobe nbd max_part=16
qemu-nbd --connect="$NBD_DEVICE" "$IMAGE_PATH"

# Wait for qemu-nbd to settle.
sleep 1

# Don't forget to cleanup, even if the script crash.
trap cleanup EXIT

# Create partition table, format partitions.
sfdisk --no-reread "$NBD_DEVICE" <<EOF
1M,500M,L,*
,,L
EOF

mkfs.ext4 "${NBD_DEVICE}p1"
mkfs.ext4 "${NBD_DEVICE}p2"

# Mount partitions, install base OS.

mount "${NBD_DEVICE}p2" /mnt
mkdir /mnt/boot
mount "${NBD_DEVICE}p1" /mnt/boot


# TODO: Remove bash
apk add -U -X $APK_MIRROR$RELEASE/main/ \
	--allow-untrusted \
	--arch="$ARCH" \
	--root=/mnt \
	--initdb \
	alpine-base alpine-conf openssh sudo tzdata gnupg haveged bash eudev

mount --bind /dev /mnt/dev
mount --bind /dev/pts /mnt/dev/pts
mount --bind /dev/shm /mnt/dev/shm
mount --bind /proc /mnt/proc
mount --bind /run /mnt/run
mount --bind /sys /mnt/sys

# Required to resolve package mirror in chroot.
cp /etc/resolv.conf /mnt/etc/resolv.conf

# Initialize networking.
run_root setup-interfaces -i << EOF
auto lo
iface lo inet loopback
EOF

cat > /mnt/etc/hosts << EOF
127.0.0.1   localhost localhost.localdomain localhost4 localhost4.localdomain4
::1         localhost localhost.localdomain localhost6 localhost6.localdomain6

EOF

# Configure package sources and update package index.
run_root setup-timezone -z UTC
if [ "$RELEASE" = "edge" ]
then
	cat >/mnt/etc/apk/repositories <<EOF
$APK_MIRROR$RELEASE/main
$APK_MIRROR$RELEASE/community
$APK_MIRROR$RELEASE/testing
EOF
else
	cat >/mnt/etc/apk/repositories <<EOF
$APK_MIRROR$RELEASE/main
$APK_MIRROR$RELEASE/community
EOF
fi

# Update package index.
run_root apk update

# Initialize base services.
for i in devfs dmesg hwdrivers mdev; do
	run_root rc-update add $i sysinit
done

for i in bootmisc hostname hwclock modules sysctl syslog acpid networking urandom haveged; do
	run_root rc-update add $i boot
done

for i in ntpd sshd crond; do
	run_root rc-update add $i default
done

for i in mount-ro killprocs savecache; do
	run_root rc-update add $i shutdown
done

# Set hostname.
run_root setup-hostname -n alpine

# Generate fstab file.
boot_uuid=$(blkid --match-tag UUID --output value "${NBD_DEVICE}p1")
root_uuid=$(blkid --match-tag UUID --output value "${NBD_DEVICE}p2")
cat >>/mnt/etc/fstab <<EOF
UUID=$boot_uuid /boot ext4 rw,relatime,data=ordered 0 2
UUID=$root_uuid /     ext4 rw,relatime,data=ordered 0 1
EOF

# Install kernel and bootloader.
run_root apk add linux-virt syslinux

dd if=/usr/share/syslinux/mbr.bin of="$NBD_DEVICE" bs=1 count=440
extlinux -i /mnt/boot

cat >/mnt/boot/extlinux.conf <<EOF
DEFAULT linux
LABEL linux
	LINUX vmlinuz-virt
	INITRD initramfs-virt
	APPEND root=UUID=$root_uuid rw modules=sd-mod,usb-storage,ext4 quiet rootfstype=ext4
EOF

# Install one-context APK and hope things works as expected.
curl -L "$ONE_CONTEXT_APK_URL" > "/mnt$ONE_CONTEXT_APK_PATH"
run_root apk add  --allow-untrusted "$ONE_CONTEXT_APK_PATH"
run_root rm "$ONE_CONTEXT_APK_PATH"

# Remove resolvconf: handled by uncloud-init.
run_root rm /etc/resolv.conf

# Make sure everything is written to disk before exiting.
sync
